==> xmame-0.37b15.2.tar.bz2: xmame-0.37b15.2/mess/vidhrdw/ssystem3.c <==
/******************************************************************************
 PeT mess@utanet.at
******************************************************************************/
#include "driver.h"
#include "artwork.h"
#include "vidhrdw/generic.h"

#include "includes/ssystem3.h"

UINT8 ssystem3_led[5]= {0};

static struct artwork_info *backdrop;

unsigned char ssystem3_palette[242][3] =
{
	{ 0,12,12 },
	{ 80,82,75 },
	{ 0,12,12 },
};

unsigned short ssystem3_colortable[1][2] = {
	{ 0, 1 },
};

void ssystem3_init_colors (unsigned char *sys_palette,
						  unsigned short *sys_colortable,
						  const unsigned char *color_prom)
{
	char backdrop_name[200];
	int used=2;

	memcpy (sys_palette, ssystem3_palette, sizeof (ssystem3_palette));
	memcpy(sys_colortable,ssystem3_colortable,sizeof(ssystem3_colortable));

    /* try to load a backdrop for the machine */
    sprintf (backdrop_name, "%s.png", Machine->gamedrv->name);

    artwork_load (&backdrop, backdrop_name, used, Machine->drv->total_colors - used);

	if (backdrop)
    {
        logerror("backdrop %s successfully loaded\n", backdrop_name);
        memcpy (&sys_palette[used * 3], backdrop->orig_palette, 
				backdrop->num_pens_used * 3 * sizeof (unsigned char));
    }
    else
    {
        logerror("no backdrop loaded\n");
    }
}

int ssystem3_vh_start(void)
{
#if 1
	/* artwork seams to need this */
    videoram_size = 6 * 2 + 24;
    videoram = (UINT8*)malloc (videoram_size);
	if (!videoram)
        return 1;

    if (backdrop)
        backdrop_refresh (backdrop);

	return generic_vh_start();
#else
	return 0;
#endif
}

void ssystem3_vh_stop(void)
{
    if (backdrop)
        artwork_free (&backdrop);
#if 1
	generic_vh_stop();
#endif
}

static const char led[]={
	"    aaaaaaaaaaaa\r"
	"  f  aaaaaaaaaa  b\r"
	"  ff  aaaaaaaa  bb\r"
	"  fff  aaaaaa  bbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	" ffff        bbbb\r"
	" ffff        bbbb\r"
	" fff          bbb\r"
    " f   gggggggg   b\r"
    "   gggggggggggg\r"
	"   gggggggggggg\r"
	" e   gggggggg   c\r"
	" eee          ccc\r"
	" eeee        cccc\r"
	" eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eee  dddddd  ccc\r"
	"ee  dddddddd  cc\r"
    "e  dddddddddd  c\r"
    "  dddddddddddd"
};

static void ssystem3_draw_7segment(struct osd_bitmap *bitmap,int value, int x, int y)
{
	int i, xi, yi, mask, color;

	for (i=0, xi=0, yi=0; led[i]; i++) {
		mask=0;
		switch (led[i]) {
		case 'a': mask=1; break;
		case 'b': mask=2; break;
		case 'c': mask=4; break;
		case 'd': mask=8; break;
		case 'e': mask=0x10; break;
		case 'f': mask=0x20; break;
		case 'g': mask=0x40; break;
		case 'h': 
			/* this is more likely wired to the separate leds */
			mask=0x80; 
			break;
		}
		
		if (mask!=0) {
			color=Machine->pens[(value&mask)?1:0];
			plot_pixel(bitmap, x+xi, y+yi, color);
			osd_mark_dirty(x+xi,y+yi,x+xi,y+yi);
		}
		if (led[i]!='\r') xi++;
		else { yi++, xi=0; }
	}
}

static const struct {
	int x,y;
} ssystem3_led_pos[5]={
	{150,123},
	{170,123},
	{200,123},
	{220,123},
	{125,123},
};

static const char* single_led=
"          c                                                                                                           1 1\r"
"   bb   ccccc   bb                                                                                                    1 1\r"
"   bb     c     bb                                                                                                    1 1\r"
" bb  bb       bb  bb                                                                                                  1 1\r"
" bb  bbbbbbbbbbb  bb                                                                                                111 111\r"
"                                                                                                                      1 1\r"
"  aa   9999999    dd                                                                                                  1 1\r"
"  aa              dd                                                                                                  1 1\r"
"     88888888888 d                                                                                                    1 1\r"
"    88         8 dddd\r"
"  8888 6666666 8 dddd\r"
"  8888 6     6\r"
"       6 777 6\r"
"       6 777 6\r"
"       6 777 6\r"
"      6  777  6\r"
"      6 77777 6\r"
"     6  77777  6                                                 2     3                                             4\r"
"     6 7777777 6                                                 2     3                                             4\r"
"    6  7777777  6                                                 2   3                                              4\r"
"    6 777777777 6                                                 2   3                                            44444\r"
"   6  777777777  6                                                 2 3                                               4\r"
"   6 77777777777 6                                                2   3                                            44444\r"
"  6       7       6                                               2   3                                              4\r"
"  6666666 7 6666666                                              2     3                                             4\r"
"          7                                                      2     3                                             4\r"
"  77777777777777777\r"
"\r"
"\r"
"\r"
" 5555555555555555555          000000   000000   00        00  0000000   00    00  00000000  00  00    00   0000000\r"
"5                   5        0000000  00000000  000      000  00000000  00    00  00000000  00  000   00  00000000\r"
"5                   5        00       00    00  0000    0000  00    00  00    00     00     00  0000  00  00\r"
"5                   5        00       00    00  00 00  00 00  00000000  00    00     00     00  00 00 00  00  0000\r"
"5                   5        00       00    00  00 00  00 00  0000000   00    00     00     00  00  0000  00    00\r"
"5                   5        0000000  00000000  00  0000  00  00        00000000     00     00  00   000  00000000\r"
" 55555555   55555555          000000   000000   00   00   00  00         000000      00     00  00    00   0000000"
;

static void ssystem3_draw_led(struct osd_bitmap *bitmap,INT16 color, int x, int y, int ch)
{
	int j, xi=0;
	for (j=0; single_led[j]; j++) {
		switch (single_led[j]) {
		default:
			if (ch==single_led[j]) {
				plot_pixel(bitmap, x+xi, y, color);
				osd_mark_dirty(x+xi,y,x+xi,y);
			}
			xi++;
			break;
		case ' ': 
			xi++;
			break;
		case '\r':
			xi=0;
			y++;
			break;				
		};
	}
}

void ssystem3_vh_screenrefresh (struct osd_bitmap *bitmap, int full_refresh)
{
	int i;

    if (backdrop)
        copybitmap (bitmap, backdrop->artwork, 0, 0, 0, 0, NULL, 
					TRANSPARENCY_NONE, 0);
	else
		fillbitmap (bitmap, Machine->pens[0], &Machine->visible_area);

	for (i=0; i<4; i++) {
		ssystem3_draw_7segment(bitmap, ssystem3_led[i]&0x7f, ssystem3_led_pos[i].x, 
							   ssystem3_led_pos[i].y);
	}

	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '0');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '1');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '2');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '3');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '4');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '5');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '6');
}

==> xmame-0.56.2.tar.bz2: xmame-0.56.2/mess/vidhrdw/ssystem3.c <==
/******************************************************************************
 PeT mess@utanet.at
******************************************************************************/
#include "driver.h"
#include "artwork.h"
#include "vidhrdw/generic.h"

#include "includes/ssystem3.h"

UINT8 ssystem3_led[5]= {0};

unsigned char ssystem3_palette[242][3] =
{
	{ 0,12,12 },
	{ 80,82,75 },
	{ 0,12,12 },
};

unsigned short ssystem3_colortable[1][2] = {
	{ 0, 1 },
};

void ssystem3_init_colors (unsigned char *sys_palette,
						  unsigned short *sys_colortable,
						  const unsigned char *color_prom)
{
	memcpy (sys_palette, ssystem3_palette, sizeof (ssystem3_palette));
	memcpy(sys_colortable,ssystem3_colortable,sizeof(ssystem3_colortable));
}

int ssystem3_vh_start(void)
{
	/* artwork seams to need this */
    videoram_size = 6 * 2 + 24;
    videoram = (UINT8*)malloc (videoram_size);
	if (!videoram)
        return 1;

	{
		char backdrop_name[200];
	    /* try to load a backdrop for the machine */
		sprintf (backdrop_name, "%s.png", Machine->gamedrv->name);
		backdrop_load(backdrop_name, 3);
	}

	return generic_vh_start();
}

void ssystem3_vh_stop(void)
{
	generic_vh_stop();
}

static const char led[]={
	"    aaaaaaaaaaaa\r"
	"  f  aaaaaaaaaa  b\r"
	"  ff  aaaaaaaa  bb\r"
	"  fff  aaaaaa  bbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	"  ffff        bbbb\r"
	" ffff        bbbb\r"
	" ffff        bbbb\r"
	" fff          bbb\r"
    " f   gggggggg   b\r"
    "   gggggggggggg\r"
	"   gggggggggggg\r"
	" e   gggggggg   c\r"
	" eee          ccc\r"
	" eeee        cccc\r"
	" eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eeee        cccc\r"
	"eee  dddddd  ccc\r"
	"ee  dddddddd  cc\r"
    "e  dddddddddd  c\r"
    "  dddddddddddd"
};

static void ssystem3_draw_7segment(struct mame_bitmap *bitmap,int value, int x, int y)
{
	int i, xi, yi, mask, color;

	for (i=0, xi=0, yi=0; led[i]; i++) {
		mask=0;
		switch (led[i]) {
		case 'a': mask=1; break;
		case 'b': mask=2; break;
		case 'c': mask=4; break;
		case 'd': mask=8; break;
		case 'e': mask=0x10; break;
		case 'f': mask=0x20; break;
		case 'g': mask=0x40; break;
		case 'h': 
			/* this is more likely wired to the separate leds */
			mask=0x80; 
			break;
		}
		
		if (mask!=0) {
			color=Machine->pens[(value&mask)?1:0];
			plot_pixel(bitmap, x+xi, y+yi, color);
			osd_mark_dirty(x+xi,y+yi,x+xi,y+yi);
		}
		if (led[i]!='\r') xi++;
		else { yi++, xi=0; }
	}
}

static const struct {
	int x,y;
} ssystem3_led_pos[5]={
	{150,123},
	{170,123},
	{200,123},
	{220,123},
	{125,123},
};

static const char* single_led=
"          c                                                                                                           1 1\r"
"   bb   ccccc   bb                                                                                                    1 1\r"
"   bb     c     bb                                                                                                    1 1\r"
" bb  bb       bb  bb                                                                                                  1 1\r"
" bb  bbbbbbbbbbb  bb                                                                                                111 111\r"
"                                                                                                                      1 1\r"
"  aa   9999999    dd                                                                                                  1 1\r"
"  aa              dd                                                                                                  1 1\r"
"     88888888888 d                                                                                                    1 1\r"
"    88         8 dddd\r"
"  8888 6666666 8 dddd\r"
"  8888 6     6\r"
"       6 777 6\r"
"       6 777 6\r"
"       6 777 6\r"
"      6  777  6\r"
"      6 77777 6\r"
"     6  77777  6                                                 2     3                                             4\r"
"     6 7777777 6                                                 2     3                                             4\r"
"    6  7777777  6                                                 2   3                                              4\r"
"    6 777777777 6                                                 2   3                                            44444\r"
"   6  777777777  6                                                 2 3                                               4\r"
"   6 77777777777 6                                                2   3                                            44444\r"
"  6       7       6                                               2   3                                              4\r"
"  6666666 7 6666666                                              2     3                                             4\r"
"          7                                                      2     3                                             4\r"
"  77777777777777777\r"
"\r"
"\r"
"\r"
" 5555555555555555555          000000   000000   00        00  0000000   00    00  00000000  00  00    00   0000000\r"
"5                   5        0000000  00000000  000      000  00000000  00    00  00000000  00  000   00  00000000\r"
"5                   5        00       00    00  0000    0000  00    00  00    00     00     00  0000  00  00\r"
"5                   5        00       00    00  00 00  00 00  00000000  00    00     00     00  00 00 00  00  0000\r"
"5                   5        00       00    00  00 00  00 00  0000000   00    00     00     00  00  0000  00    00\r"
"5                   5        0000000  00000000  00  0000  00  00        00000000     00     00  00   000  00000000\r"
" 55555555   55555555          000000   000000   00   00   00  00         000000      00     00  00    00   0000000"
;

static void ssystem3_draw_led(struct mame_bitmap *bitmap,INT16 color, int x, int y, int ch)
{
	int j, xi=0;
	for (j=0; single_led[j]; j++) {
		switch (single_led[j]) {
		default:
			if (ch==single_led[j]) {
				plot_pixel(bitmap, x+xi, y, color);
				osd_mark_dirty(x+xi,y,x+xi,y);
			}
			xi++;
			break;
		case ' ': 
			xi++;
			break;
		case '\r':
			xi=0;
			y++;
			break;				
		};
	}
}

void ssystem3_vh_screenrefresh (struct mame_bitmap *bitmap, int full_refresh)
{
	int i;

	for (i=0; i<4; i++) {
		ssystem3_draw_7segment(bitmap, ssystem3_led[i]&0x7f, ssystem3_led_pos[i].x, 
							   ssystem3_led_pos[i].y);
	}

	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '0');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '1');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '2');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '3');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '4');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '5');
	ssystem3_draw_led(bitmap, Machine->pens[ssystem3_led[4]&8?1:0], 
				 ssystem3_led_pos[4].x, ssystem3_led_pos[4].y, '6');
}
